﻿--[[Copyright (C) 2008 by Fritos

	This program is free software; you can redistribute it and--or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.]]
	
Dailies = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0", "FuBarPlugin-2.0", "AceDB-2.0")

local Tablet = AceLibrary("Tablet-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Crayon = AceLibrary("LibCrayon-3.0")
local Quixote = AceLibrary("LibQuixote-2.0")
local Deformat = AceLibrary("Deformat-2.0")
local Abacus = AceLibrary("LibAbacus-3.0")
local Sink = AceLibrary("LibSink-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Dailies")
local BZ = AceLibrary("LibBabble-Zone-3.0"):GetLookupTable()
local BF = AceLibrary("LibBabble-Faction-3.0"):GetLookupTable()

Sink:Embed(Dailies)

Dailies.hasIcon  = "Interface\\Addons\\DailiesQuestTracker\\Icon.tga"
Dailies.hasNoText  = false;
Dailies.title = L["Dailies Quest Tracker"];
Dailies.hideMenuTitle = true;
Dailies.cannotDetachTooltip = false;
Dailies:RegisterDB("DailiesDB");
Dailies.clickableTooltip = true;

-- Tradeskill Holder
DailiesTradeskills = {}
-- Scheduled reset time for dailies (i.e 3 am Server Time)
DailiesReset = 4
-- Toggle Seasonal Filter off by default unless Holiday Dailies are availiable
ShowSeasonal = false
GrabbedChannelList = false

-- Random Dailies values
DailiesKnownNotCompleted = Crayon:Red(L["Not Completed"])
DailiesKnownCompleted = Crayon:Green(L["Completed"])

-- Comm Setup
Dailies.CommPrefix = "DQTComm";
Dailies:SetCommPrefix(Dailies.CommPrefix);
Dailies.OnCommReceive = {}
DailiesRoster = {}
Dailies.UniqueNames = {};
DailiesCommHolder = {}

DialogOffComms = Crayon:Gold(L["You're turning "]) .. Crayon:Red(L["OFF"]) .. Crayon:Gold(L[" communications for "]) .. Crayon:Green(L["Dailies Quest Tracker"]) .. Crayon:Gold(L[".\n\n Clicking "]).. Crayon:Green(L["YES"])
	.. Crayon:Gold(L[" will reload your UI to remove the channel and you will no longer send and/or receive the Random Dailies!\n\nAre you sure you would like to do this?"])
DialogOnComms = Crayon:Gold(L["You're turning "]) .. Crayon:Green(L["ON"]) .. Crayon:Gold(L[" communications for "]) .. Crayon:Green(L["Dailies Quest Tracker"]) .. Crayon:Gold(L[".\n\n Clicking "]).. Crayon:Green(L["YES"])
	.. Crayon:Gold(L[" will reload your UI to set-up the channel, please make sure your "]) .. Crayon:Green(L["CUSTOM"]) .. Crayon:Gold(L[" channels are set before continuing.\n\nAre you sure you would like to do this?"])

-- Reputation Table holder
local RepFactionsHolder = {
		["None"] = 0,
		["BGPvP"] = 0,
		["Cooking"] = 0,
		["Fishing"] = 0,
		["Holiday"] = 0,
		["HInstance"] = 0,
		["NInstance"] = 0,
		["World PvP"] = 0,
		["Shattered Sun Offensive"] = nil,
		["Netherwing"] = nil,
		["Ogri'la"] = nil,
		["Sha'tari Skyguard"] = nil,
}

-- Reputation Faction Colors
local RepFactionColors = {
	[0] = {r = 0.5, g = 0.5, b = 0.5},		-- Unknown
	[1] = {r = 0.55, g = 0, b = 0},			-- hated
	[2] = {r = 1, g = 0, b = 0},			-- hostile
	[3] = {r = 1, g = 0.55, b = 0},			-- unfriendly
	[4] = {r = 0.75, g = 0.75, b = 0.75},	-- neutral
	[5] = {r = 1, g = 1, b = 1},			-- friendly
	[6] = {r = 0, g = 1, b = 0},			-- honored
	[7] = {r = 0.25, g = 0.4, b = 0.9},		-- revered
	[8] = {r = 0.6, g = 0.2, b = 0.8},		-- exalted	
}

 -- Coverter from RGB to HEX
local function RGBPercToHex(r, g, b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end
 
 -- Output Messages to Chatframe
function Dailies:Debug(...)
	if self.db.profile and self.db.profile.DebugEnabled == true then
		self:Print(...);
	end
end

function Dailies:ResetAllComms()
	ReloadUI();
end

function Dailies:OnInitialize()
englishFaction, _ = UnitFactionGroup("Player")
PlayerRace, _ = UnitRace("Player");

-- Turn Comm Channels On/Off Dialog
StaticPopupDialogs["RESET_COMMS"] = {
	text = "%s",
	button1 = "YES",
	button2 = "NO",
	OnAccept = function()
		self.db.char.DailyCommSet  = not self.db.char.DailyCommSet;		
		Dailies:ResetAllComms();
	end,
	OnCancel = function (_, reason)
		if reason == "clicked" then
			SetDailyCommSet = self.db.char.DailyCommSet;
		end
	end,
	timeout = 0,
	whileDead = 1,
	showAlert = 1
};


local config = {
	type = 'group',
	args = {
	}
}

Dailies:RegisterChatCommand({"/DQT", "/Dailies"}, config)

DailiesGroups = {
	["BGPvP"] = {group = "BGPvP", title = L["Battleground"] .." ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["Cooking"] = {group ="Cooking", title = GetSpellInfo(2550) .. " ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["Fishing"] = {group ="Fishing", title = GetSpellInfo(7620) .. " ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["Holiday"] = {group ="Holiday", title = L["Holiday"] .. " ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["HInstance"] = {group ="HInstance", title = L["Heroic Instance"] .. " ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["NInstance"] = {group ="NInstance", title = L["Dungeon Instance"] .." ["..Crayon:Green(L["Random Daily"]).."]", ValueMax = 1},
	["Netherwing"] = {group ="Netherwing", title = BF["Netherwing"], ValueMax = 8},
	["Ogri'la"] = {group ="Ogri'la", title = BF["Ogri'la"], ValueMax = 2},
	["Sha'tari Skyguard"] = {group ="Sha'tari Skyguard", title = BF["Sha'tari Skyguard"], ValueMax = 4},
	["Shattered Sun Offensive"] = {group = "Shattered Sun Offensive", title = BF["Shattered Sun Offensive"], ValueMax = 17},
	["World PvP"] = {group ="World PvP", title = L["World PvP"], ValueMax = 3}
}

if englishFaction == "Alliance" then
	-- Regular Instance
	DailyQuestsList[L["Wanted: Shattered Hand Centurions"]] = {icon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Centurion.tga", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "NInstance", SID = "None:0", ["rep"] = {BF["Honor Hold"]..":250", BF["The Consortium"]..":250"}, ["rewards"] = {"29460:1"}, gold = 163900} --Shattered Halls
	-- Heroic Instance
	DailyQuestsList[L["Wanted: Nazan's Riding Crop"]] = {icon = "Interface\\Icons\\INV_Misc_Crop_01", loc = BZ["Hellfire Ramparts"], area = BZ["Hellfire Ramparts"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Ramparts
	DailyQuestsList[L["Wanted: Keli'dan's Feathered Stave"]] = {icon = "Interface\\Icons\\INV_Staff_Goldfeathered_01", loc = BZ["The Blood Furnace"], area = BZ["The Blood Furnace"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Blood Furnace
	DailyQuestsList[L["Wanted: Bladefist's Seal"]] = {icon = "Interface\\Icons\\INV_SummerFest_Symbol_Medium", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "HInstance", SID = "Honor Hold:6", ["rep"] = {BF["Honor Hold"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Shattered Halls
	--Fishing [Patch 2.4]
	DailyQuestsList[L["Crocolisks in the City"]] = {icon = "Interface\\Icons\\Ability_Hunter_Pet_Crocolisk", area = BZ["Stormwind City"], group = "Fishing", SID = "None:0", ["rewards"] = {"35348:1"}}
	-- Hellfire World PvP
	DailyQuestsList[L["Hellfire Fortifications"]] = {icon = "Interface\\Icons\\INV_Misc_Token_HonorHold", group = "World PvP", loc = BZ["Hellfire Peninsula"], area = BZ["Hellfire Peninsula"], SID = "Honor Hold:4", ["rep"] = {BF["Honor Hold"]..":150"}, ["rewards"] = {"24579:3"}, gold = 11350}
	-- Alliance Nagrand PvP [Patch 2.4]
	DailyQuestsList[L["Spirits of Auchindoun"]] = {icon = "Interface\\Icons\\INV_Jewelry_FrostwolfTrinket_04", group = "World PvP", loc = BZ["Terokkar Forest"], area = BZ["Terokkar Forest"], SID = "None:0", ["honor"] = 210, gold = 119900}
	-- Alliance Terokkar PvP [Patch 2.4]
	DailyQuestsList[L["In Defense of Halaa"]] = {icon = "Interface\\Icons\\INV_Misc_Rune_08", group = "World PvP", loc = BZ["Nagrand"], area = BZ["Nagrand"], SID = "None:0", ["honor"] = 210, gold = 119900}
	BGHonor = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Alliance.tga"
	
elseif englishFaction == "Horde" then
	-- Regular Instance
	DailyQuestsList[L["Wanted: Shattered Hand Centurions"]] = {icon = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Centurion.tga", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "NInstance", SID = "None:0", ["rep"] = {BF["Thrallmar"]..":250", BF["The Consortium"]..":250"}, ["rewards"] = {"29460:1"}, gold = 163900} --Shattered Halls
	-- Heroic Instance
	DailyQuestsList[L["Wanted: Nazan's Riding Crop"]] = {icon = "Interface\\Icons\\INV_Misc_Crop_01", loc = BZ["Hellfire Ramparts"], area = BZ["Hellfire Ramparts"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Ramparts
	DailyQuestsList[L["Wanted: Keli'dan's Feathered Stave"]] = {icon = "Interface\\Icons\\INV_Staff_Goldfeathered_01", loc = BZ["The Blood Furnace"], area = BZ["The Blood Furnace"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Blood Furnace
	DailyQuestsList[L["Wanted: Bladefist's Seal"]] = {icon = "Interface\\Icons\\INV_SummerFest_Symbol_Medium", loc = BZ["The Shattered Halls"], area = BZ["The Shattered Halls"], group = "HInstance", SID = "Thrallmar:6", ["rep"] = {BF["Thrallmar"]..":350", BF["The Consortium"]..":350"}, ["rewards"] = {"29434:2"}, gold = 246000} --Shattered Halls
	--Fishing [Patch 2.4]
	DailyQuestsList[L["Crocolisks in the City"]] = {icon = "Interface\\Icons\\Ability_Hunter_Pet_Crocolisk", area = BZ["Orgrimmar"], group = "Fishing", SID = "None:0", ["rewards"] = {"35348:1"}}
	-- Hellfire World PvP
	DailyQuestsList[L["Hellfire Fortifications"]] = {icon = "Interface\\Icons\\INV_Misc_Token_Thrallmar", group = "World PvP", loc = BZ["Hellfire Peninsula"], area = BZ["Hellfire Peninsula"], SID = "Thrallmar:4", ["rep"] = {BF["Thrallmar"]..":150"}, ["rewards"] = {"24581:3"}, gold = 11350}
	-- Horde Nagrand PvP [Patch 2.4]
	DailyQuestsList[L["Spirits of Auchindoun"]] = {icon = "Interface\\Icons\\INV_Jewelry_FrostwolfTrinket_04", group = "World PvP", loc = BZ["Terokkar Forest"], area = BZ["Terokkar Forest"], SID = "None:0", ["honor"] = 210, gold = 119900}
	-- Horde Terokkar PvP [Patch 2.4]
	DailyQuestsList[L["Enemies, Old and New"]] = {icon = "Interface\\Icons\\INV_Misc_Rune_08", group = "World PvP", loc = BZ["Nagrand"], area = BZ["Nagrand"], SID = "None:0", ["honor"] = 210, gold = 119900}
	BGHonor = "Interface\\Addons\\DailiesQuestTracker\\Icons\\Horde.tga"
end

Dailies:RegisterDefaults('profile', {
	DebugEnabled = false,
	QuestsVisible = true,
	hideDetails = {
		["BGPvP"] = true,
		["Cooking"] = true,
		["Holiday"] = true,
		["Fishing"] = true,
		["HInstance"] = true,
		["NInstance"] = true,
		["Netherwing"] = true,
		["Ogri'la"] = true,
		["Sha'tari Skyguard"] = true,
		["Shattered Sun Offensive"] = true,
		["World PvP"] = true,
		},
})

Dailies:RegisterDefaults('char', {
	DailyQuestsAccepted = {},
	DailyQuestsCompleted = {},
	DailyQuestsObjectives = {},
	TooltipPlacement = "Right",
	DailyCommSet = false,
	LocationShow = true,
	HideHints = false,
	HideDailyList = false,
	Filters = {
		["BGPvP"] = false,
		["Cooking"] = false,
		["Fishing"] = false,
		["Holiday"] = false,
		["HInstance"] = false,			
		["NInstance"] = false,
		["Netherwing"] = false,
		["Ogri'la"] = false,
		["Sha'tari Skyguard"] = false,
		["Shattered Sun Offensive"] = false,
		["World PvP"] = false,
	},
	GroupValues = {
		["BGPvP"] = 0,
		["Cooking"] = 0,
		["Fishing"] = 0,
		["Holiday"] = 0,
		["HInstance"] = 0,			
		["NInstance"] = 0,
		["Netherwing"] = 0,
		["Ogri'la"] = 0,
		["Sha'tari Skyguard"] = 0,
		["Shattered Sun Offensive"] = 0,
		["World PvP"] = 0,
	},
})

Dailies:RegisterEvent("CHAT_MSG_SKILL", "CHAT_MSG_SKILL")
Dailies:RegisterEvent("CHAT_MSG_SYSTEM", "DailiesQuest_Tracker")
Dailies:RegisterEvent("UNIT_QUEST_LOG_CHANGED", "DailiesUpdate")
Dailies:RegisterEvent("PLAYER_ALIVE", "PLAYER_ALIVE")
Dailies:RegisterEvent("ADDON_LOADED", "ADDON_LOADED")
Dailies:RegisterEvent("Quixote_Quest_Complete", "QuestObjectives")
Dailies:RegisterEvent("ZONE_CHANGED", "ZONE_CHANGED")
Dailies:RegisterEvent("ZONE_CHANGED_INDOORS", "ZONE_CHANGED")
Dailies:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZONE_CHANGED")
Dailies:RegisterEvent("GOSSIP_SHOW", "GOSSIP_SHOW")
Dailies:RegisterEvent("QUEST_DETAIL", "QUEST_DETAIL")

Dailies:RegisterEvent("TurnOnComms", "TurnOnComms");
Dailies:RegisterEvent("CHANNEL_COUNT_UPDATE", "GetChannelMembers");
Dailies:RegisterEvent("CHANNEL_ROSTER_UPDATE", "GetChannelMembers");

Dailies:RegisterEvent("SendTheDailies", "SendTheDailies")

Dailies:DailiesSeasonal()
Dailies:PLAYER_ALIVE()

Dailies:SetSinkStorage(self.db.profile)

StaticPopupDialogs["RESET_COMMS"].OnShow = function()
		for index = 1, STATICPOPUP_NUMDIALOGS, 1 do
			local frame = getglobal("StaticPopup"..index);
			local alertIcon = getglobal(frame:GetName().."AlertIcon");
			alertIcon:SetTexture("Interface\\Addons\\DailiesQuestTracker\\Icons\\Network.tga")
		end
	end;
	
StaticPopupDialogs["ABANDON_QUEST"].OnAccept = function() Dailies:AbandonedQuest(GetAbandonQuestName()); AbandonQuest(); PlaySound("igQuestLogAbandonQuest"); end
StaticPopupDialogs["ABANDON_QUEST_WITH_ITEMS"].OnAccept = function() Dailies:AbandonedQuest(GetAbandonQuestName()); AbandonQuest(); PlaySound("igQuestLogAbandonQuest"); end
end

-- Check dates to add in Seasonal Dailies
function Dailies:DailiesSeasonal()
	-- Midsummer Fire Festival
	if (DailiesMONTH == 6 and DailiesDAY >= 21) and (DailiesMONTH == 6 and DailiesDAY <= 30) then
			self.db.char.Filters["Holiday"] = false; ShowSeasonal = true; DailiesGroups["Holiday"]["ValueMax"] = 4
			DailyQuestsList[L["More Torch Tossing"]] = {icon = "Interface\\Icons\\INV_Torch_Lit", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
			DailyQuestsList[L["More Torch Catching"]] = {icon = "Interface\\Icons\\INV_TradeskillItem_03", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
			DailyQuestsList[L["Striking Back"]] = {icon = "Interface\\Icons\\INV_Misc_Gem_AzureDraenite_02", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:10"},  gold = 106900}
			DailyQuestsList[L["Summon Ahune"]] = {icon = "Interface\\Icons\\Spell_FrostResistanceTotem_01", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, gold = 119900}
		
	elseif (DailiesMONTH == 7 and DailiesDAY >= 1) and (DailiesMONTH == 7 and DailiesDAY <= 5) then
			self.db.char.Filters["Holiday"] = false; ShowSeasonal = true; DailiesGroups["Holiday"]["ValueMax"] = 4
			DailyQuestsList[L["More Torch Tossing"]] = {icon = "Interface\\Icons\\INV_Torch_Lit", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
			DailyQuestsList[L["More Torch Catching"]] = {icon = "Interface\\Icons\\INV_TradeskillItem_03", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:5"},  gold = 119900}
			DailyQuestsList[L["Striking Back"]] = {icon = "Interface\\Icons\\INV_Misc_Gem_AzureDraenite_02", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, ["rewards"] = {"23247:10"},  gold = 106900}
			DailyQuestsList[L["Summon Ahune"]] = {icon = "Interface\\Icons\\Spell_FrostResistanceTotem_01", group = "Holiday", SID = "Holiday:0", dates = {starting = "6/21", ending = "7/05"}, gold = 119900}

	-- Brewfest
	elseif (DailiesMONTH == 10 and DailiesDAY >= 2) and (DailiesMONTH == 10 and DailiesDAY <= 16) then
		self.db.char.Filters["Holiday"] = false; ShowSeasonal = true; DailiesGroups["Holiday"]["ValueMax"] = 2
		if englishFaction == "Alliance" then
			DailyQuestsList[L["Bark for the Barleybrews!"]] = {icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}
			DailyQuestsList[L["Bark for the Thunderbrews!"]] = {icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}	
		elseif englishFaction == "Horde" then
			DailyQuestsList[L["Bark for Drohn's Distillery!"]] = {icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}
			DailyQuestsList[L["Bark for T'chali's Voodoo Brewery!"]] = {icon = "Interface\\Icons\\INV_Belt_05", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/02", ending = "10/16"}, ["rewards"] = {"33455:40"}}
		end
		
	-- Headless Horseman (Hallow's End)
	elseif (DailiesMONTH == 10 and DailiesDAY >= 18) and (DailiesMONTH == 10 and DailiesDAY <= 31) then
		self.db.char.Filters["Holiday"] = false; ShowSeasonal = true; DailiesGroups["Holiday"]["ValueMax"] = 1
		DailyQuestsList[L["Call the Headless Horseman"]] = {icon = "Interface\\Icons\\INV_Misc_Bag_28_Halloween", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  gold = 7500}

	elseif (DailiesMONTH == 11 and DailiesDAY >= 1) and (DailiesMONTH == 11 and DailiesDAY <= 2) then
		self.db.char.Filters["Holiday"] = false; ShowSeasonal = true; DailiesGroups["Holiday"]["ValueMax"] = 1
		DailyQuestsList[L["Call the Headless Horseman"]] = {icon = "Interface\\Icons\\INV_Misc_Bag_28_Halloween", group = "Holiday", SID = "Holiday:0", dates = {starting = "10/18", ending = "11/02"},  gold = 7500}
	else
		self.db.char.Filters["Holiday"] = true; ShowSeasonal = false
	end
end

-- Check to see if USER wants unfolding of groups and unfold when necessary
function Dailies:ZONE_CHANGED()
	local DailiesZoneName = GetRealZoneText(); local DailiesSubZoneName = GetSubZoneText();
	local DailiesInInstance, DailiesInstanceType = IsInInstance(); local DailiesDifficulty = GetInstanceDifficulty()
	
	if self.db.char.LocationShow == true then
		if (Tablet20Frame) and (Tablet20Frame:IsVisible()) then
			return
		else
			for GroupName, GroupText in pairsByKeys (DailiesGroups) do
				self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] = true
			end
			
			for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
				if DailyQuestsList[index]["area"] == DailiesZoneName then
					self.db.profile.hideDetails[DailyQuestsList[index]["group"]] = false
				end
			end
			
			if DailiesInInstance and DailiesInstanceType == "pvp"  then
				self.db.profile.hideDetails["BGPvP"] = false
			end		
			
			if DailiesInInstance then
				if DailiesInstanceType == "party" and DailiesDifficulty == 2 then
					for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
						if DailyQuestsList[index]["area"] == DailiesZoneName then
							self.db.profile.hideDetails["NInstance"] = false
						end
						
						if DailyQuestsList[index]["area"] == DailiesZoneName then
							self.db.profile.hideDetails["HInstance"] = false
						end
					end
				end
			else
				if DailiesInstanceType == "party" and DailiesDifficulty == 1 then
					for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
						if DailyQuestsList[index]["area"] == DailiesZoneName then
							self.db.profile.hideDetails["NInstance"] = false
						end
					end
				end
			end
			Dailies:UpdateTooltip();
		end
	end
end

-- Check to see if Fubar is loaded
function Dailies:ADDON_LOADED(arg1)
	if (arg1 == "LibFuBarPlugin-3.0") or (arg1 == "LibFuBarPlugin-2.0") then
		Dailies:Debug(L["Fubar Plugin Loaded!"])
		LibRockFuBar = true
		FuBarFontSize = FuBar:GetFontSize()
	end
 end
 
 -- Hack for when someone Abandons a daily, so we can reset it in the FuBar tooltip
function Dailies:AbandonedQuest()
	local AbandonedName = GetAbandonQuestName();
	for index, value in pairs (Dailies.db.char.DailyQuestsAccepted) do
		if index == AbandonedName then
			Dailies.db.char.DailyQuestsAccepted[index] = nil;
			Dailies.db.char.DailyQuestsObjectives[index] = nil;
		end
	end
end

-- Gold Sorter
function goldByKeys (t, f)
	local TempHolder = {}
	for n in pairs (t) do 
		table.insert (TempHolder, n) 
	end
	
	table.sort(TempHolder, function(a, b)
		if t[a]["gold"] == t[b]["gold"] then
			return a < b
		else
			return (t[a]["gold"] or 0) > (t[b]["gold"] or 0)
		end
	end)
		
--[[	table.sort(TempHolder, function(a, b)
		return (t[a]["gold"] or 0) > (t[b]["gold"] or 0)
	end)]]
		
	local i = 0
	return function ()
	i = i + 1
	return TempHolder[i], t[TempHolder[i]]
	end
end

-- Title Sorter
function pairsByKeys (t, f)
	local TempHolder = {}
	for n in pairs (t) do 
		table.insert (TempHolder, n) 
	end
	table.sort (TempHolder, f)
	local i = 0
	return function ()
	i = i + 1
	return TempHolder[i], t[TempHolder[i]]
	end
end

-- Checks to see if the USER has finished all the objectives in a quest
function Dailies:QuestObjectives(QuestNameObjectives, QID)
	if Dailies.db.char.DailyQuestsAccepted[QuestNameObjectives] then
		Dailies:Debug(L["You have completed the objectives for the quest!"])
		Dailies.db.char.DailyQuestsObjectives[QuestNameObjectives] = "Turn-In"
	end
	Dailies:UpdateTooltip();
end

-- Check for EVENT triggers
function Dailies:DailiesQuest_Tracker(msg)
	-- /script Dailies:DailiesQuest_Tracker("You can complete 25 daily quests today.")
	-- /script Dailies:DailiesQuest_Tracker("Quest accepted: Super Hot Stew")
	-- /script Dailies:DailiesQuest_Tracker("Quest accepted: Felblood Fillet")

	_, ChatFrameFontSize, _, _, _, _, _, _, _ = GetChatWindowInfo(1)
	local QuestName = Deformat(msg, ERR_QUEST_COMPLETE_S)
	local QuestNameAccepted = Deformat(msg, ERR_QUEST_ACCEPTED_S)
	local DailiesQuestReset = Deformat(msg, DAILY_QUESTS_REMAINING)
	
	if DailiesQuestReset == GetMaxDailyQuests() then
		Dailies:Debug(L["Dailies Have Reset!"])
		ResetText = Crayon:Green(L["Dailies have reset for the day."])
		DEFAULT_CHAT_FRAME:AddMessage ("\124TInterface\\Addons\\DailiesQuestTracker\\Icons\\Reset.tga:" .. ChatFrameFontSize .. "\124t" .. ResetText); 
		self:Pour(ResetText, nil, nil, nil, nil, nil, "OUTLINE", nil, "NOTIFY", "Interface\\Addons\\DailiesQuestTracker\\Icons\\Reset.tga")
		self.db.char.DailyQuestsCompleted = {};
		DailyHInstance = nil; DailyNInstance = nil; DailyCooking = nil; DailyFishing = nil; DailyBGPvP = nil;
		DailiesCommHolder = {};
		self.db.char.GroupValues = {
			["BGPvP"] = 0,
			["Cooking"] = 0,
			["Fishing"] = 0,
			["Holiday"] = 0,
			["HInstance"] = 0,			
			["NInstance"] = 0,
			["Netherwing"] = 0,
			["Ogri'la"] = 0,
			["Sha'tari Skyguard"] = 0,
			["Shattered Sun Offensive"] = 0,
			["World PvP"] = 0,			
		}
		Dailies:Update();
	end
	
	for QuestAcceptedIndex, QuestAcceptedValue in pairs(DailyQuestsList) do
		if (QuestNameAccepted == QuestAcceptedIndex) then
			Dailies:Debug(L["Quest Accepted!"])
			local QuestHour, QuestMinute = GetGameTime(); local QuestPM = floor(QuestHour / 12) == 1; QuestHour = mod(QuestHour, 12); if QuestHour == 0 then QuestHour = 12 end;
			QuestTimeAccepted = string.format("%d" .. Crayon:Gold(":") .. "%02d" .. Crayon:Gold(" %s"), QuestHour, QuestMinute, QuestPM and "PM" or "AM");
			Dailies.db.char.DailyQuestsAccepted[QuestNameAccepted] = QuestTimeAccepted
			self:Pour(QuestNameAccepted .. L[" accepted at "] .. QuestTimeAccepted, nil, nil, nil, nil, nil, "OUTLINE", nil, "NOTIFY", DailyQuestsList[QuestNameAccepted]["icon"])
			Dailies:ZONE_CHANGED()
		end
	end
	
	for QuestCompletedIndex, QuestCompletedValue in pairs(DailyQuestsList) do
		if (QuestName == QuestCompletedIndex) then
			Dailies:Debug(L["Quest Completed!"])
			local QuestHour, QuestMinute = GetGameTime(); local QuestPM = floor(QuestHour / 12) == 1; QuestHour = mod(QuestHour, 12); if QuestHour == 0 then QuestHour = 12 end;
			QuestTimeCompleted = string.format("%d" .. Crayon:Gold(":") .. "%02d" .. Crayon:Gold(" %s"), QuestHour, QuestMinute, QuestPM and "PM" or "AM");
			Dailies.db.char.DailyQuestsCompleted[QuestName] = QuestTimeCompleted; Dailies.db.char.DailyQuestsAccepted[QuestCompletedIndex] = nil; Dailies.db.char.DailyQuestsObjectives[QuestCompletedIndex] = nil
			self:Pour(QuestName .. L[" completed at "] .. QuestTimeCompleted, nil, nil, nil, nil, nil, "OUTLINE", nil, "NOTIFY", DailyQuestsList[QuestName]["icon"])
			Dailies:QuestValue(QuestName)
		end
	end
	Dailies:Update();
end

-- Holder and Counter for groups
function Dailies:QuestValue(QuestName)
	Dailies:Debug(L["Adding Group Values!"])
	CookingValue = Dailies.db.char.GroupValues["Cooking"]
	if DailyQuestsList[QuestName]["group"] == "Cooking" then CookingValue = CookingValue + 1 end; Dailies.db.char.GroupValues["Cooking"] = CookingValue
	
	WorldPvPValue = Dailies.db.char.GroupValues["World PvP"]
	if DailyQuestsList[QuestName]["group"] == "World PvP" then WorldPvPValue = WorldPvPValue + 1 end; Dailies.db.char.GroupValues["World PvP"] = WorldPvPValue

	FishingValue = Dailies.db.char.GroupValues["Fishing"]
	if DailyQuestsList[QuestName]["group"] == "Fishing" then FishingValue = FishingValue + 1 end; Dailies.db.char.GroupValues["Fishing"] = FishingValue	
	
	NetherwingValue = Dailies.db.char.GroupValues["Netherwing"]
	if DailyQuestsList[QuestName]["group"] == "Netherwing" then NetherwingValue = NetherwingValue + 1 end; Dailies.db.char.GroupValues["Netherwing"] = NetherwingValue
	
	SSOValue = Dailies.db.char.GroupValues["Shattered Sun Offensive"]
	if DailyQuestsList[QuestName]["group"] == "Shattered Sun Offensive" then SSOValue = SSOValue + 1 end; Dailies.db.char.GroupValues["Shattered Sun Offensive"] = SSOValue

	OgrilaValue = Dailies.db.char.GroupValues["Ogri'la"]
	if DailyQuestsList[QuestName]["group"] == "Ogri'la" then OgrilaValue = OgrilaValue + 1 end; Dailies.db.char.GroupValues["Ogri'la"] = OgrilaValue
		
	SkyguardValue = Dailies.db.char.GroupValues["Sha'tari Skyguard"]
	if DailyQuestsList[QuestName]["group"] == "Sha'tari Skyguard" then SkyguardValue = SkyguardValue + 1 end; Dailies.db.char.GroupValues["Sha'tari Skyguard"] = SkyguardValue
	
	HolidayValue = Dailies.db.char.GroupValues["Holiday"]
	if DailyQuestsList[QuestName]["group"] == "Holiday" then HolidayValue = HolidayValue + 1 end; Dailies.db.char.GroupValues["Holiday"] = HolidayValue	

	BGPvPValue = Dailies.db.char.GroupValues["BGPvP"]
	if DailyQuestsList[QuestName]["group"] == "BGPvP" then BGPvPValue = BGPvPValue + 1 end; Dailies.db.char.GroupValues["BGPvP"] = BGPvPValue	

	NInstanceValue = Dailies.db.char.GroupValues["NInstance"]
	if DailyQuestsList[QuestName]["group"] == "NInstance" then NInstanceValue = NInstanceValue + 1 end; Dailies.db.char.GroupValues["NInstance"] = NInstanceValue	

	HInstanceValue = Dailies.db.char.GroupValues["HInstance"]
	if DailyQuestsList[QuestName]["group"] == "HInstance" then HInstanceValue = HInstanceValue + 1 end; Dailies.db.char.GroupValues["HInstance"] = HInstanceValue		
end

function Dailies:DailiesUpdate()
	DQMax = GetMaxDailyQuests();
	DQComplete = GetDailyQuestsCompleted();
	Dailies:Update()
end

function Dailies:TurnOnComms()
	if (self.db.char.DailyCommSet == false) then
		return
	else
	Dailies:RegisterComm(Dailies.CommPrefix, "GLOBAL")
	Dailies:RegisterComm(Dailies.CommPrefix, "WHISPER")
	Dailies:ScheduleRepeatingEvent("SendTheDailies", 15)
	
	_, ChatFrameFontSize, _, _, _, _, _, _, _ = GetChatWindowInfo(1)
	CommText = Crayon:Green(L[" You have joined the GLOBAL AceComm channel."])
		DEFAULT_CHAT_FRAME:AddMessage ("\124TInterface\\Addons\\DailiesQuestTracker\\Icons\\Comm.tga:" .. ChatFrameFontSize .. "\124t" .. CommText);		
	end
end


function Dailies:OnEnable()
	DQComplete = GetDailyQuestsCompleted();
	DQMax = GetMaxDailyQuests();
	
	if DQComplete == 0 then
		self.db.char.DailyQuestsCompleted = {};
		self.db.char.GroupValues = {
			["BGPvP"] = 0,
			["Cooking"] = 0,
			["Fishing"] = 0,
			["Holiday"] = 0,
			["HInstance"] = 0,
			["NInstance"] = 0,			
			["Netherwing"] = 0,
			["Ogri'la"] = 0,
			["Sha'tari Skyguard"] = 0,
			["Shattered Sun Offensive"] = 0,
			["World PvP"] = 0,
		}
	end
	
	hour, minute = GetGameTime();
	DQStartHour = hour;
	DQStartMiniute = minute;
	SetDailyCommSet = self.db.char.DailyCommSet;
	
	Dailies:ScheduleEvent("TurnOnComms", 10)
end

-- Gets the Members in the AceComm Channel to communicate the known dailies
function Dailies:GetChannelMembers()
	channelCount = GetNumDisplayChannels();
	
	for i=1, channelCount do
		channel, header, collapsed, channelNumber, count, active, category = GetChannelDisplayInfo(i);
		
		if (count) and (table.getn(DailiesRoster) ~= count) then
			GrabbedChannelList = false
		else
			GrabbedChannelList = true
		end
							
		if (count) then
			if channel == "AceComm" then
				if (GrabbedChannelList == false) then
					DailiesRoster = {}
					SetSelectedDisplayChannel(i)
					Dailies:Debug(L["Old Count: "] .. table.getn(DailiesRoster) .. L[" New Count: "] .. count)
					
					for y=1, count do
						local name, owner, moderator, muted, voiceEnabled = GetChannelRosterInfo(GetSelectedDisplayChannel(), y)
						DailiesRoster[y] = name
						Dailies:Debug(DailiesRoster[y])
					end
				end
			end
		end
	end
end

-- Sends the known dailies
function Dailies:SendTheDailies()
	if DailyHInstance then
		Dailies:SendTheQuest("HInstance", DailiesCommHolder["HInstance"])
	end

	if DailyNInstance then
		Dailies:SendTheQuest("NInstance", DailiesCommHolder["NInstance"])
	end

	if DailyCooking then
		Dailies:SendTheQuest("Cooking", DailiesCommHolder["Cooking"])
	end

	if DailyFishing then
		Dailies:SendTheQuest("Fishing", DailiesCommHolder["Fishing"])
	end

	if DailyBGPvP then
		Dailies:SendTheQuest("BGPvP", DailiesCommHolder["BGPvP"])
	end	
	
	Dailies:SendTheUser("MEMBER", UnitName("player"))
end

function Dailies:SendTheUser(CommUser, SenderName)
	for i = 1, table.getn(DailiesRoster) do
		if AceLibrary("AceComm-2.0"):IsUserInChannel(DailiesRoster[i], "GLOBAL") then
			Dailies:SendCommMessage("WHISPER", DailiesRoster[i], CommUser, SenderName)
		end
	end
end

function Dailies:SendTheQuest(Instance, questName)
	for i = 1, table.getn(self.UniqueNames) do
		if (self.UniqueNames[i] and self.UniqueNames[i] ~= UnitName("player")) then
			if AceLibrary("AceComm-2.0"):IsUserInChannel(self.UniqueNames[i], "GLOBAL") then
				Dailies:Debug (L["Sending Dailies to: "] .. self.UniqueNames[i])
				Dailies:SendCommMessage("WHISPER", self.UniqueNames[i], Instance, questName)
			else
				Dailies:Debug(L["Removing User: "] .. self.UniqueNames[i])
				table.remove(self.UniqueNames, i);
			end
		end
	end
end

function Dailies:GetUniqueNames(SenderName)
	local doInsert = true;
	for i=1,table.getn(self.UniqueNames) do
		if self.UniqueNames[i] == SenderName then
			doInsert = false;
		end
	end	
	
	if doInsert then
		table.insert(self.UniqueNames, SenderName);
		Dailies:Debug(table.getn(self.UniqueNames))
	end
end

-- Receives the dailies and collects the information
function Dailies.OnCommReceive:HInstance (prefix, sender, distribution, questName)
	Dailies:Debug (L["Heroic: "] .. sender)
--	Dailies:GetUniqueNames(sender)
	DailiesCommHolder["HInstance"] = questName; DailyHInstance = questName; DailiesKnown = true; Dailies:UpdateTooltip()
end

function Dailies.OnCommReceive:NInstance (prefix, sender, distribution, questName)
	Dailies:Debug (L["Dungeon: "] .. sender)
--	Dailies:GetUniqueNames(sender)
	DailiesCommHolder["NInstance"] = questName; DailyNInstance = questName; DailiesKnown = true; Dailies:UpdateTooltip()
end

function Dailies.OnCommReceive:Cooking (prefix, sender, distribution, questName)
	Dailies:Debug (GetSpellInfo(2550) .. ": " .. sender)
--	Dailies:GetUniqueNames(sender)
	DailiesCommHolder["Cooking"] = questName; DailyCooking = questName; DailiesKnown = true; Dailies:UpdateTooltip()
end

function Dailies.OnCommReceive:Fishing (prefix, sender, distribution, questName)
	Dailies:Debug (GetSpellInfo(7620) .. ": "  .. sender)
--	Dailies:GetUniqueNames(sender)
	DailiesCommHolder["Fishing"] = questName; DailyFishing = questName; DailiesKnown = true; Dailies:UpdateTooltip()
end

function Dailies.OnCommReceive:BGPvP (prefix, sender, distribution, questName)
	Dailies:Debug (L["Battleground: "] .. sender)
--	Dailies:GetUniqueNames(sender)
	DailiesCommHolder["BGPvP"] = questName; DailyBGPvP = questName; DailiesKnown = true; Dailies:UpdateTooltip()
end

function Dailies.OnCommReceive:MEMBER (prefix, sender, distribution, UserName)
	local doInsert = true;
	for i=1,table.getn(self.UniqueNames) do
		if self.UniqueNames[i] == UserName then
			doInsert = false;
		end
	end	
	
	if doInsert then
		table.insert(self.UniqueNames, UserName);
		Dailies:Debug(L["Adding User: "] .. UserName)
	end
end

-- Check if Aquired new tradeskills and update gathering quests you can do
function Dailies:CHAT_MSG_SKILL(msg)
	local TradeskillGained = Deformat(msg, ERR_SKILL_GAINED_S)
	if TradeskillGained ==  GetSpellInfo(8613) or TradeskillGained == GetSpellInfo(2575) or TradeskillGained == GetSpellInfo(9134) then
		SSOCounter = 0;	TradeskillCounter = 0
		hasHerbalism = nil; hasMining = nil; hasSkinning = nil;
		Dailies:PLAYER_ALIVE()
	end
end

function Dailies:PLAYER_ALIVE()
	DQMax = GetMaxDailyQuests();
	DQComplete = GetDailyQuestsCompleted();
	-- Cycle through Tradeskills
	numSkills = GetNumSkillLines();
	if TradeskillCounter == 0 then
		for i=1, numSkills do
			skillName, _, _, skillRank, _, _, _, _, _, _, _, _, _ = GetSkillLineInfo(i)
			DailiesTradeskills[skillName] = skillRank
			
			if skillName == GetSpellInfo(9134) then
				DailyQuestsList[L["Gaining the Advantage"]] = {icon = "Interface\\Icons\\INV_MISC_DUST_03", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}
				DailyQuestsList[L["Netherdust Pollen"]] = {icon = "Interface\\Icons\\INV_Enchant_DustSoul", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(9134), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900} --Herbalism
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasHerbalism = GetSpellInfo(9134);
			elseif skillName == GetSpellInfo(2575) then
				DailyQuestsList[L["Gaining the Advantage"]] = {icon = "Interface\\Icons\\INV_MISC_DUST_03", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}			
				DailyQuestsList[L["Nethercite Ore"]] = {icon = "Interface\\Icons\\INV_Ore_Ethernium_01", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(2575), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900} --Mining
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasMining = GetSpellInfo(2575);
			elseif skillName == GetSpellInfo(8613) then
				DailyQuestsList[L["Gaining the Advantage"]] = {icon = "Interface\\Icons\\INV_MISC_DUST_03", group = "Shattered Sun Offensive", area = BZ["Outland"], SID = "Shattered Sun Offensive:4", ["rep"] = {BF["Shattered Sun Offensive"]..":250"}, ["rewards"] = {"18253:1"}, gold = 163900}
				DailyQuestsList[L["Nethermine Flayer Hide"]] = {icon = "Interface\\Icons\\INV_Misc_Rune_12", group = "Netherwing", area = BZ["Shadowmoon Valley"], SID = "Netherwing:4", prof = GetSpellInfo(8613), ["rep"] = {BF["Netherwing"]..":250"}, gold = 119900}  --Skinning
				DailiesGroups["Netherwing"]["ValueMax"] = DailiesGroups["Netherwing"]["ValueMax"] + 1
				hasSkinning = GetSpellInfo(8613);
			end
			
			if SSOCounter == 0 then
				if (hasHerbalism or hasMining or hasSkinning) then
					DailiesGroups["Shattered Sun Offensive"]["ValueMax"] = DailiesGroups["Shattered Sun Offensive"]["ValueMax"] + 1
					SSOCounter = 1
				end
			end
			TradeskillCounter = 1
		end	
	end
	Dailies:UpdateTooltip();
end

function Dailies:OnTextUpdate()
	Dailies:SetIcon("Interface\\Addons\\DailiesQuestTracker\\Icons\\Numbers\\" .. (GetMaxDailyQuests() - GetDailyQuestsCompleted()) .. ".tga")
	
	DQColor = Crayon:GetThresholdHexColor((GetMaxDailyQuests() - GetDailyQuestsCompleted()), GetMaxDailyQuests())
	Dailies:SetText(" " .. Crayon:Colorize(DQColor, (GetMaxDailyQuests() - GetDailyQuestsCompleted())) .. "/" .. Crayon:Green(GetMaxDailyQuests()));
end

-- Dailies Quest Tracker options menu
function Dailies:OnMenuRequest(level, value, inTooltip)
    if not inTooltip or inTooltip then
		if level == 1 then
			-- Dailies Config Title
			Dewdrop:AddLine(
				'text', L["Dailies Config"],
				'isTitle', true,
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icon.tga",
				'iconWidth', 12,
				'iconHeight', 12
			)

			Dewdrop:AddLine(
				'text', L["Random Dailies Communication"],
				'checked', self.db.char.DailyCommSet,
				'func', function ()
					SetDailyCommSet  = not SetDailyCommSet
					if (SetDailyCommSet  == true) then
						StaticPopup_Show ("RESET_COMMS", DialogOnComms)
					elseif (SetDailyCommSet  == false) then
						StaticPopup_Show ("RESET_COMMS", DialogOffComms)
					end
				end,
				'tooltipTitle', L["Random Dailies Communication"],
				'tooltipText', L["This toggles turning communications on/off for Dailies Quest Tracker."]
			)
			
			-- Show All Dailies
			Dewdrop:AddLine(
				'text', L["Show All Dailies"],
				'checked', self.db.profile.QuestsVisible,
				'func', function ()
					self.db.profile.QuestsVisible  = not self.db.profile.QuestsVisible ;
					if (self.db.profile.QuestsVisible  == true) then
						self.db.profile.QuestsVisible  = true;
					else
						self.db.profile.QuestsVisible  = false;
					end
					Dailies:UpdateTooltip();
				end,
				'tooltipTitle', L["Show All Dailies"],
				'tooltipText', L["Shows all dailies that you are able to do."]
			)
			
			-- HINTS
			Dewdrop:AddLine(
				'text', L["Hide "] .. Crayon:Green(L["[HINTS]"]),
				'checked', self.db.char.HideHints,
				'func', function ()
					self.db.char.HideHints  = not self.db.char.HideHints
				end,
				'tooltipTitle', L["Hide "] .. Crayon:Green(L["[HINTS]"]),
				'tooltipText', L["Hides the "] .. Crayon:Green(L["[HINTS]"]) .. L[" in Dailies Quest Tracker."]
			)

			Dewdrop:AddLine(
				'text', L["Hide Random Dailies List"],
				'checked', self.db.char.HideDailyList,
				'disabled', not self.db.char.DailyCommSet,
				'func', function ()
					self.db.char.HideDailyList  = not self.db.char.HideDailyList
				end,
				'tooltipTitle', L["Hide Random Dailies List"],
				'tooltipText', L["Hides the random dailies list in Dailies Quest Tracker."]
			)			
			
			-- Location unfold
			Dewdrop:AddLine(
				'text', L["Auto Unfold Groups"],
				'checked', self.db.char.LocationShow,
				'func', function ()
					self.db.char.LocationShow  = not self.db.char.LocationShow
				end,
				'tooltipTitle', L["Auto Unfold Groups"],
				'tooltipText', L["Automatically unfolds groups when you are in the zone or instance of those quests."]
			)			

			-- Spacer
			Dewdrop:AddSeparator()
			
			-- Filters
			Dewdrop:AddLine(
				'text', L["Filters"],
				'hasArrow', true,
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\Filter.tga",
				'value', "Filters",
				'tooltipTitle', L["Filters"],
				'tooltipText', L["Lets you filter out groups you have no interest in watching."]
			)
			
			-- Tooltip Placement
			Dewdrop:AddLine(
				'text', L["Rewards Tooltip Placement"],
				'hasArrow', true,
				'value', "Rewards",
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\Tooltip.tga",
				'tooltipTitle',  L["Rewards Tooltip Placement"],
				'tooltipText', L["Places the rewards tooltip on either the Left or Right of Dailies Quest Tracker."]
			)			
				
			-- Tooltip Placement
			Dewdrop:AddLine(
				'text', L["Show Me The Money!"],
				'hasArrow', true,
				'value', "MoneyRewards",
				'icon', "Interface\\Icons\\INV_Misc_Coin_02",
				'tooltipTitle',  L["Show Me The Money!"],
				'tooltipText', L["Makes a list of the highest monetary value of the Dailies."]
			)			
			
			
			-- Scrolling Combat Text Title
			Dewdrop:AddLine(
			'text', L["Scrolling Combat Text"],
			'hasArrow', true,
			'value', "SCT",
			'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\SCT.tga",
			'tooltipTitle', L["Scrolling Combat Text"],
			'tooltipText', L["Displays notifications if you have specific addon's installed."]
			)

			-- Spacer
			Dewdrop:AddSeparator()

			Dewdrop:AddLine(
				'text', L["Debug"],
				'checked', self.db.profile.DebugEnabled,
				'func', function ()
					self.db.profile.DebugEnabled  = not self.db.profile.DebugEnabled
				end,
				'tooltipTitle', L["Debug Dailies Quest Tracker"],
				'tooltipText', L["Turns the Debugger on/off for Dailies Quest Tracker."]
			)			
			
		elseif level == 2 then
			if value == "Rewards" then
				Dewdrop:AddLine(
				'text', L["Left"],
				'checked', self.db.char.TooltipPlacement == "Right",
				'func', function ()
					RightTooltip = true; LeftTooltip = nil; 
					self.db.char.TooltipPlacement = "Right"
				end,
				'isRadio', true
				)
				
				Dewdrop:AddLine(
				'text', L["Right"],
				'checked', self.db.char.TooltipPlacement == "Left",
				'func', function ()
					RightTooltip = nil; LeftTooltip = true; 
					self.db.char.TooltipPlacement = "Left"
				end,
				'isRadio', true
				)
			end

			if value == "Filters" then
				Dewdrop:AddLine(
				'text', L["Battleground"],
				'checked', self.db.char.Filters["BGPvP"],
				'func', function ()
					self.db.char.Filters["BGPvP"] = not self.db.char.Filters["BGPvP"]
				end,
				'isRadio', true
				)
				
				Dewdrop:AddLine(
				'text', GetSpellInfo(2550),
				'checked', self.db.char.Filters["Cooking"],
				'func', function ()
					self.db.char.Filters["Cooking"] = not self.db.char.Filters["Cooking"]				
				end,
				'isRadio', true
				)

				Dewdrop:AddLine(
				'text', L["Heroic Instance"],
				'checked', self.db.char.Filters["HInstance"],
				'func', function ()
					self.db.char.Filters["HInstance"] = not self.db.char.Filters["HInstance"]				
				end,
				'isRadio', true
				)
		
				Dewdrop:AddLine(
				'text', GetSpellInfo(7620),
				'checked', self.db.char.Filters["Fishing"],
				'func', function ()
					self.db.char.Filters["Fishing"] = not self.db.char.Filters["Fishing"]				
				end,
				'isRadio', true
				)
				
				if ShowSeasonal == true then
					Dewdrop:AddLine(
					'text', L["Holiday"],
					'checked', self.db.char.Filters["Holiday"],
					'func', function ()
						self.db.char.Filters["Holiday"] = not self.db.char.Filters["Holiday"]				
					end,
					'isRadio', true
					)
				end
				
				Dewdrop:AddLine(
				'text', L["Dungeon Instance"],
				'checked', self.db.char.Filters["NInstance"],
				'func', function ()
					self.db.char.Filters["NInstance"] = not self.db.char.Filters["NInstance"]				
				end,
				'isRadio', true
				)

				if RepFactionsHolder["Netherwing"] then
					Dewdrop:AddLine(
					'text', BF["Netherwing"],
					'checked', self.db.char.Filters["Netherwing"],
					'func', function ()
						self.db.char.Filters["Netherwing"] = not self.db.char.Filters["Netherwing"]				
					end,
					'isRadio', true
					)
				end

				if RepFactionsHolder["Ogri'la"] then
					Dewdrop:AddLine(
					'text', BF["Ogri'la"],
					'checked', self.db.char.Filters["Ogri'la"],
					'func', function ()
						self.db.char.Filters["Ogri'la"] = not self.db.char.Filters["Ogri'la"]				
					end,
					'isRadio', true
					)
				end

				if RepFactionsHolder["Sha'tari Skyguard"] then
					Dewdrop:AddLine(
					'text', BF["Sha'tari Skyguard"],
					'checked', self.db.char.Filters["Sha'tari Skyguard"],
					'func', function ()
						self.db.char.Filters["Sha'tari Skyguard"] = not self.db.char.Filters["Sha'tari Skyguard"]				
					end,
					'isRadio', true
					)
				end
				
				if RepFactionsHolder["Shattered Sun Offensive"] then
					Dewdrop:AddLine(
					'text', BF["Shattered Sun Offensive"],
					'checked', self.db.char.Filters["Shattered Sun Offensive"],
					'func', function ()
						self.db.char.Filters["Shattered Sun Offensive"] = not self.db.char.Filters["Shattered Sun Offensive"]				
					end,
					'isRadio', true
					)
				end		

				Dewdrop:AddLine(
				'text', L["World PvP"],
				'checked', self.db.char.Filters["World PvP"],
				'func', function ()
					self.db.char.Filters["World PvP"] = not self.db.char.Filters["World PvP"]				
				end,
				'isRadio', true
				)				
			
			end
			
			if value == "SCT" then
				options = self:GetSinkAce2OptionsDataTable().output
				Dewdrop:FeedAceOptionsTable(options, 1)
			end

			if value == "MoneyRewards" then
				for QuestName, GroupText in goldByKeys (DailyQuestsList) do
					 if Dailies.db.char.DailyQuestsCompleted[QuestName] then
						QuestStatus = Crayon:Green(QuestName)
					else
						QuestStatus = Crayon:White(QuestName)
					end
					
					if DailyQuestsList[QuestName]["group"] == "HInstance" then
						QuestLocation = Crayon:White(L["Heroic Instance: "])
					elseif DailyQuestsList[QuestName]["group"] == "NInstance" then
						QuestLocation = Crayon:White(L["Dungeon Instance: "])
					elseif DailyQuestsList[QuestName]["group"] == "BGPvP" then
						QuestLocation = Crayon:White(L["Battleground: "])
					else
						QuestLocation = Crayon:White(L["Location: "])
					end
					
					if DailyQuestsList[QuestName]["gold"] then
					Dewdrop:AddLine(
						'text', Crayon:Gold("[") .. Abacus:FormatMoneyFull(DailyQuestsList[QuestName]["gold"], true) .. Crayon:Gold("] ") .. QuestStatus,
						'icon', DailyQuestsList[QuestName]["icon"],
						'tooltipTitle', QuestStatus,
						'tooltipText', QuestLocation .. DailyQuestsList[QuestName]["area"] .. "\n" .. Crayon:White(L["Monetary Value: "]) .. Abacus:FormatMoneyExtended(DailyQuestsList[QuestName]["gold"], true)
						)
					end
				end
			end		
		end
	end
	Dailies:UpdateTooltip();
end

function Dailies:SecondsToTime(seconds, noSeconds, notAbbreviated)
	local time = "";
	local count = 0;
	local tempTime;
	seconds = floor(seconds);
	if ( seconds >= 86400  ) then
		tempTime = floor(seconds / 86400);
		if ( notAbbreviated ) then
			time = tempTime.." "..GetText("DAYS", nil, tempTime);
		else
			time = tempTime.." "..GetText("DAYS_ABBR", nil, tempTime);
		end
		seconds = mod(seconds, 86400);
		count = count + 1;
	end
	if ( seconds >= 3600  ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		tempTime = floor(seconds / 3600);
		if ( notAbbreviated ) then
			time = time..tempTime..":"
		else
			time = time..tempTime..":"
		end
		seconds = mod(seconds, 3600);
		count = count + 1;
	end
	if ( count < 2 and seconds >= 60  ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		tempTime = floor(seconds / 60);
		if ( notAbbreviated ) then
			time = time..tempTime..":"
		else
			time = time..tempTime..":"
		end
		seconds = mod(seconds, 60);
		count = count + 1;
	end
	if ( count < 2 and seconds > 0 and not noSeconds ) then
		if ( time ~= "" ) then
			time = time.." ";
		end
		seconds = format("%d", seconds);
		if ( notAbbreviated ) then
			time = time..seconds..":"
		else
			time = time..seconds..":"
		end
	end
	return time;
end


function Dailies:OnTooltipUpdate()
	-- Cycle through the USER's known factions and hold them in a table to see if groups should be displayed
	for factionIndex = 1, GetNumFactions() do
	-- name, description, standingId, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched = GetFactionInfo(factionIndex)
		FactionName, _, standingID, _, _, _, _, _, isHeader, isCollapsed, _ = GetFactionInfo(factionIndex)
		RepFactionsHolder[FactionName] = standingID
	end

	if DailiesReset == 4 then
		DailiesResetSeconds = GetQuestResetTime() + 3600
	else
		DailiesResetSeconds = GetQuestResetTime()
	end
	
	local DailiesResetTimeHour, DailiesResetTimeMin = (":"):split(Dailies:SecondsToTime(DailiesResetSeconds, true, 1))
	local DailiesResetTimeHour = tonumber(DailiesResetTimeHour)
	local DailiesResetTimeMin = tonumber(DailiesResetTimeMin)

	if DailiesResetTimeHour then
		if DailiesResetTimeHour > 1 then
			HoursFormat = L["Hours"]
		else
			HoursFormat = L["Hour"]
		end
	end
	
	if DailiesResetTimeMin then
		if DailiesResetTimeMin > 1 then
			MinutesFormat = L["Minutes"]
		else
			MinutesFormat = L["Minute"]
		end
	end
	
	if DailiesResetSeconds >= 3600 then
		DailiesIconError = false
		if DailiesResetTimeMin then
			DailiesText = (L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeHour .. " " .. HoursFormat) .. L[" and "] .. Crayon:Green(DailiesResetTimeMin .. " " .. MinutesFormat) .. L["."])
		else
			DailiesText = (L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeHour .. " " .. HoursFormat) .. L["."])
		end
	else
		if DailiesResetTimeMin then
			DailiesText = (L["Dailies are scheduled to reset in "] .. Crayon:Green(DailiesResetTimeMin .. " " .. MinutesFormat) .. L["."])
		else
			DailiesText = Crayon:Red(L["[ERROR]: "]) .. "|cffAA0000" .. L["Reset time from realm is not valid."] .. "|r"
			DailiesIconError = true
		end
	end

	local DQTooltipColor = Crayon:GetThresholdHexColor((GetMaxDailyQuests() - GetDailyQuestsCompleted()), GetMaxDailyQuests())
	if ( DQComplete < DQMax ) then
		-- You can complete XX daily quests today
		AmountText = L["You can complete "] .. Crayon:Colorize(DQTooltipColor, (GetMaxDailyQuests() - GetDailyQuestsCompleted())) ..L[" daily quests today."]
	else
		-- You cannot complete any more daily quests today
		AmountText = L["You cannot complete any more daily quests today."]
	end
	
	Tablet:SetTitle(L["Dailies Quest Tracker for "] .. DailesDATE)
	
	cat = Tablet:AddCategory(
		'columns', 1,
		'showWithoutChildren', true,
		'hideBlankLine', true
	 )

	if table.getn(self.UniqueNames) <= 0 then
		UniqueUsers = Crayon:Red(L["None"])
	else
		UniqueUsers = Crayon:Green(table.getn(self.UniqueNames))
	end
	
	if self.db.char.DailyCommSet == true then
		UserCommSet = Crayon:Green(L["ON"])
		cat:AddLine(
		'text', Crayon:Silver(L["Random Dailies Communication: "]) .. UserCommSet ..  Crayon:Gold(" | ") .. Crayon:Silver(L["DQT Users: "]) .. UniqueUsers,
		'justify', "CENTER"
		)
	else
		UserCommSet = Crayon:Red(L["OFF"])
		cat:AddLine(
		'text', Crayon:Silver(L["Random Dailies Communication: "]) .. UserCommSet,
		'justify', "CENTER"
	)
	end
	
		if self.db.char.DailyQuestsCompleted then
		for GroupName, GroupText in pairsByKeys (DailiesGroups) do
			if self.db.profile.QuestsVisible == true and self.db.char.Filters[GroupName] == false and RepFactionsHolder[GroupName] then
				MinColor = Crayon:GetThresholdHexColor(self.db.char.GroupValues[DailiesGroups[GroupName]["group"]], DailiesGroups[GroupName]["ValueMax"])
				cat = Tablet:AddCategory(
					'text', DailiesGroups[GroupName]["title"],
					'textR', 1, 'textG', 0.8, 'textB', 0,
					'text2', "[" .. Crayon:Colorize(MinColor, self.db.char.GroupValues[GroupName]) .. "/" .. Crayon:Green(DailiesGroups[GroupName]["ValueMax"]) .. "]",
					'text2R', 1, 'text2G', 0.8, 'text2B', 0,
					'columns', 2,
					'showWithoutChildren', true,
					'hideBlankLine', true,
					'func', function() self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] = not self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] end,
					'hasCheck', true,
					'checked', true,
					'checkIcon', self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
				)
				
				for index, value in pairsByKeys (DailyQuestsList) do
					if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then		
						if DailyQuestsList[index]["group"] == GroupName then
							if self.db.char.DailyQuestsObjectives[index] == "Turn-In" then RColor= 0; GColor= 1; BColor = 0; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = L["Turn-In"]
								elseif self.db.char.DailyQuestsCompleted[index] then RColor= 1; GColor= 1; BColor = 1; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = self.db.char.DailyQuestsCompleted[index]
								elseif self.db.char.DailyQuestsAccepted[index] then RColor= 0; GColor= 1; BColor = 0; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = L["Accepted"]
								elseif self.db.char.GroupValues[DailiesGroups[GroupName]["group"]] ==  DailiesGroups[GroupName]["ValueMax"] then RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0; CompletedText = L["Group Max"]
								else RColor= 0.5; GColor= 0.5; BColor = 0.5; RIcon= 0.5; GIcon= 0.5; BIcon = 0.5;
								if GetDailyQuestsCompleted() == GetMaxDailyQuests() then RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0; CompletedText = L["Dailies Max"] end

								if DailyQuestsList[index]["dates"] then
									CompletedText = DailyQuestsList[index]["dates"].starting .." - ".. DailyQuestsList[index]["dates"].ending
								else 
									CompletedText = DailyQuestsList[index]["loc"]
								end
							end
							
							local ReputationName, ReputationID = (":"):split(DailyQuestsList[index]["SID"]); ReputationID = tonumber(ReputationID)
							if (index and RepFactionsHolder[ReputationName]) and (ReputationID <= RepFactionsHolder[ReputationName])  then
								cat:AddLine(
									'text', " " .. index,
									'textR', RColor,
									'textG', GColor,
									'textB', BColor,
									'indentation', 12,
									'text2', CompletedText,
									'text2R', RColor,
									'text2G', GColor,
									'text2B', BColor,
									'hasCheck', true,
									'checked', true,
									'checkIcon', DailyQuestsList[index]["icon"],
									'checkColorR', RIcon,
									'checkColorG', BIcon,
									'checkColorB', GIcon,
									'func', 'OnQuestClick',
									'arg1', self,
									'arg2', index
								)
							end
						end
					end
				end
				if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then cat:AddLine () end
			
			elseif self.db.char.GroupValues[DailiesGroups[GroupName]["group"]] > 0  and self.db.char.Filters[GroupName] == false then
				MinColor = Crayon:GetThresholdHexColor(self.db.char.GroupValues[DailiesGroups[GroupName]["group"]], DailiesGroups[GroupName]["ValueMax"])
				cat = Tablet:AddCategory(
					'text', DailiesGroups[GroupName]["title"],
					'textR', 1, 'textG', 0.8, 'textB', 0,
					'text2', "[" .. Crayon:Colorize(MinColor, self.db.char.GroupValues[GroupName]) .. "/" .. Crayon:Green(DailiesGroups[GroupName]["ValueMax"]) .. "]",
					'text2R', 1, 'text2G', 0.8, 'text2B', 0,
					'columns', 2,
					'showWithoutChildren', true,
					'hideBlankLine', true,
					'func', function() self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] = not self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] end,
					'hasCheck', true,
					'checked', true,
					'checkIcon', self.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] and "Interface\\Buttons\\UI-PlusButton-Up" or "Interface\\Buttons\\UI-MinusButton-Up"
				)
				
				for index, value in pairsByKeys (self.db.char.DailyQuestsCompleted) do
					if self.db.char.DailyQuestsObjectives[index] == "Turn-In" then RColor= 0; GColor= 1; BColor = 0; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = L["Turn-In"]
						elseif self.db.char.DailyQuestsCompleted[index] then RColor= 1; GColor= 1; BColor = 1; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = self.db.char.DailyQuestsCompleted[index]
						elseif self.db.char.DailyQuestsAccepted[index] then RColor= 0; GColor= 1; BColor = 0; RIcon= 1; GIcon= 1; BIcon = 1; CompletedText = L["Accepted"]
						elseif self.db.char.GroupValues[DailiesGroups[GroupName]["group"]] ==  DailiesGroups[GroupName]["ValueMax"] then RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0; CompletedText = L["Group Max"]
						elseif GetDailyQuestsCompleted() == GetMaxDailyQuests() then RColor= 1; GColor= 0; BColor = 0; RIcon= 1; GIcon= 0; BIcon = 0; CompletedText = L["Dailies Max"]
						else RColor= 0.5; GColor= 0.5; BColor = 0.5; RIcon= 0.5; GIcon= 0.5; BIcon = 0.5
					end
					if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then		
						if DailyQuestsList[index]["group"] == GroupName then
							cat:AddLine(
								'text', " " .. index,
								'textR', RColor,
								'textG', GColor,
								'textB', BColor,
								'indentation', 12,
								'text2', CompletedText,
								'text2R', 1,
								'text2G', 1,
								'text2B', 1,
								'hasCheck', true,
								'checked', true,
								'checkIcon', DailyQuestsList[index]["icon"],
								'checkColorR', 1,
								'checkColorG', 1,
								'checkColorB', 1,
								'func', 'OnQuestClick',
								'arg1', self,
								'arg2', index
							)
						end
					end
				end
				if Dailies.db.profile.hideDetails[DailiesGroups[GroupName]["group"]] == false then cat:AddLine () end
			end
		end
	
	if (self.db.char.DailyCommSet == true) then
		if (self.db.char.HideDailyList == false) then
			if (DailiesKnown == nil) then
			-- Random Dailies Report Warning
				cat:AddLine(
					'text', " "
				)
				
				cat:AddLine(
					'text', Crayon:Red(L["[MESSAGE]: "]) .. "|cffAA0000" .. L["One moment while gathering dailies from other recipients!"] .. "|r",
					'hasCheck', true,
					'checked', true,
					'checkIcon', "Interface\\HelpFrame\\HotIssueIcon"
				)

				cat:AddLine(
					'text', " "
				)		

			-- Random Dailies Report List
			elseif (DailiesKnown ~= nil) then
				-- Spacer	
				cat = Tablet:AddCategory(
					'columns', 1,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )

				cat:AddLine(
					'text', " "
				)
				
				cat:AddLine(
				'text', L["The "] .."["..Crayon:Green(L["Random Dailies"]).."]" .. L[" for "] .. DailesDATE .. L[" are:"],
				'justify', "CENTER"
				)
				
				cat = Tablet:AddCategory(
					'columns', 3,
					'showWithoutChildren', true,
					'hideBlankLine', true
				 )

				if DailyBGPvP then
					if Dailies.db.char.DailyQuestsCompleted[DailyBGPvP] then
						DailiesKnownBGPvP = DailiesKnownCompleted
					else
						DailiesKnownBGPvP = DailiesKnownNotCompleted
					end
					cat:AddLine(
					'text', Crayon:White(DailyBGPvP),
					'text2', "[" .. DailiesKnownBGPvP .."]",
					'text3', "[" .. Crayon:Green(L["Daily Battleground"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyBGPvP]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyBGPvP,
					'arg3', L[" The Battleground Daily is: "]				
					)
				end
				
				if DailyCooking then
					if Dailies.db.char.DailyQuestsCompleted[DailyCooking] then
						DailiesKnownCooking = DailiesKnownCompleted
					else
						DailiesKnownCooking = DailiesKnownNotCompleted
					end
					cat:AddLine(
					'text', Crayon:White(DailyCooking),
					'text2', "[" .. DailiesKnownCooking .."]",
					'text3', "[" .. Crayon:Green(L["Daily Cooking"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyCooking]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyCooking,
					'arg3', L[" The Cooking Daily is: "]
					)
				end

				if DailyFishing then
					if Dailies.db.char.DailyQuestsCompleted[DailyFishing] then
						DailiesKnownFishing = DailiesKnownCompleted
					else
						DailiesKnownFishing = DailiesKnownNotCompleted
					end
					cat:AddLine(
					'text', Crayon:White(DailyFishing),
					'text2', "[" .. DailiesKnownFishing .."]",
					'text3', "[" .. Crayon:Green(L["Daily Fishing"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyFishing]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyFishing,
					'arg3', L[" The Fishing Daily is: "]				
					)
				end
				
				if DailyHInstance then
					if Dailies.db.char.DailyQuestsCompleted[DailyHInstance] then
						DailiesKnownHInstance = DailiesKnownCompleted
					else
						DailiesKnownHInstance = DailiesKnownNotCompleted
					end
					cat:AddLine(
					'text', Crayon:White(DailyHInstance),
					'text2', "[" .. DailiesKnownHInstance .."]",
					'text3', "[" .. Crayon:Green(L["Daily Heroic"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyHInstance]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyHInstance,
					'arg3', L[" The Heroic Daily is: "]
					)
				end
				
				if DailyNInstance then
					if Dailies.db.char.DailyQuestsCompleted[DailyNInstance] then
						DailiesKnownNInstance = DailiesKnownCompleted
					else
						DailiesKnownNInstance = DailiesKnownNotCompleted
					end
					cat:AddLine(
					'text', Crayon:White(DailyNInstance),
					'text2', "[" .. DailiesKnownNInstance .."]",
					'text3', "[" .. Crayon:Green(L["Daily Dungeon"]) .."]",
					'hasCheck', true,
					'checked', true,
					'checkIcon', DailyQuestsList[DailyNInstance]["icon"],
					'func', 'QuestClickLink',
					'arg1', self,
					'arg2', DailyNInstance,
					'arg3', L[" The Dungeon Daily is: "]				
					)
				end
				
				cat:AddLine(
					'text', " "
				)			
			end
		end
	end
	
	cat = Tablet:AddCategory(
		'columns', 1,
		'showWithoutChildren', true,
		'hideBlankLine', true
	)
	end

	-- Displays how many dailies are left for the day
	cat:AddLine(
		'text', AmountText,
		'justify', "CENTER"
	)
	
	-- Displays when the dailies will reset
	cat:AddLine(
		'text', DailiesText,
		'justify', "CENTER",
		'checkIcon', "Interface\\HelpFrame\\HotIssueIcon"
	)
	
	-- Tooltip DISCLAIMER
	if UnitLevel("player") < 70 then
		cat:AddLine(
			'text', Crayon:Red(L["[DISCLAIMER]: "]) .. "|cffAA0000" .. L["Reputation and rewards are based on a level 70 character."] .. "|r",
			'hasCheck', true,
			'checked', true,
			'checkIcon', "Interface\\HelpFrame\\HotIssueIcon"
		)
	end
	
	if self.db.char.HideHints == false then
		-- FuBar Left-Click HINT
		cat:AddLine(
			'text', Crayon:Green(L["[HINT]: "]) .. Crayon:White(L["Left-Click"]) .. L[" on the FuBar icon to open the Quest Log Window."],
			'textR', 0,
			'textG', 0.5,
			'textB', 0,
			'hasCheck', true,
			'checked', true,
			'checkIcon', "Interface\\GossipFrame\\AvailableQuestIcon"
		)
		
		-- Tooltip Left-Click HINT
		cat:AddLine(
			'text', Crayon:Green(L["[HINT]: "]) .. Crayon:White(L["Left-Click"]) .. L[" on a quest title to see rewards associated with that quest."],
			'textR', 0,
			'textG', 0.5,
			'textB', 0,
			'hasCheck', true,
			'checked', true,
			'checkIcon', "Interface\\GossipFrame\\AvailableQuestIcon"
		)
		
		-- Tooltip Itemlink HINT
		cat:AddLine(
			'text', Crayon:Green(L["[HINT]: "]) .. Crayon:White(L["Left-Click"]) .. L[" on a reward "] .. Crayon:White("[itemlink]") ..L[" to paste it into the chatbox."],
			'textR', 0,
			'textG', 0.5,
			'textB', 0,
			'hasCheck', true,
			'checked', true,
			'checkIcon', "Interface\\GossipFrame\\AvailableQuestIcon"
		)
	end
end

function Dailies:OnQuestClick(QuestTitleValue)
	-- If USER is using FuBar then make the tooltips look consistant
	if (LibRockFuBar == true) then
		FontIconSize = FuBar.db.profile.tooltip.fontSizePercent
		FuBarFontSize = FuBar:GetFontSize()
	else
		FuBarFontSize = 10
		FontIconSize = 1
	end
	
	-- Make a new tooltip if the USER clicks on a quest title
	Dewdrop:Open(Tablet20Frame,
    'children', function()
	    Dewdrop:AddLine(
            'text', QuestTitleValue .. L[" Rewards:"],
			'textHeight', FontIconSize * 12, 
			'isTitle', true,
			'icon', DailyQuestsList[QuestTitleValue]["icon"],
			'iconWidth', FontIconSize * 14,
			'iconHeight', FontIconSize * 14
        )
		
		-- If ITEMS are awarded, add them to the tooltip
		if DailyQuestsList[QuestTitleValue]["rewards"] then
			for Reward, RewardType in pairs(DailyQuestsList[QuestTitleValue]["rewards"]) do
			-- Grab itemlink and quantity from quest and break it apart
			local RewardType, RewardAmount = (":"):split(RewardType); RewardAmount = tonumber(RewardAmount)
			if RewardAmount < 2 then RewardAmount = "" else RewardAmount = (" x" .. RewardAmount) end
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(RewardType); 
			if itemLink == nil then GameTooltip:SetHyperlink("item:".. RewardType); end
			
			if itemLink then
		        Dewdrop:AddLine(
					'text', itemLink .. RewardAmount,
					'textHeight', FontIconSize * 12, 
					'icon', itemTexture,
					'iconWidth', FontIconSize * 14,
					'iconHeight', FontIconSize * 14,
					'tooltipFunc', self.SetTooltip,
					'tooltipArg1', GameTooltip, 'tooltipArg2', RewardType,
					'func', 'itemClickLink',
					'arg1', self,
					'arg2', itemLink,
					'arg3', RewardAmount
		        )
			end
			end
		end

		-- If HONOR awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["honor"] then
			Dewdrop:AddLine(
				'text', "~" .. DailyQuestsList[QuestTitleValue]["honor"] .. L[" Honor Points"],
				'textHeight', FontIconSize * 12, 
				'icon', BGHonor,
				'iconWidth', FontIconSize * 14,
				'iconHeight', FontIconSize * 14
	        )
		end

		-- If REPUTATION awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["rep"] then
			-- Cycle through the USER's known factions and hold them in a table to see if a quest should be displayed
			for factionIndex = 1, GetNumFactions() do
				FactionName, _, standingID, _, _, earnedValue, _, _, _, _, _ = GetFactionInfo(factionIndex)
				RepFactionsHolder[FactionName] = standingID
			end
	
			for RepReward, RepRewardType in pairs(DailyQuestsList[QuestTitleValue]["rep"]) do
			-- Grab faction name and reputation from quest and break it apart
			local RepRewardType, RepAmount = (":"):split(RepRewardType)
			-- Check to see if the PLAYER is HUMAN and multiply Reputation by 10%
			if PlayerRace == "Human" then RepAmount = ceil(RepAmount + (RepAmount * .10)) else RepAmount = RepAmount end
			-- If Repuation Standing is nil then set it to UNKNOWN
			if RepFactionsHolder[RepRewardType] == nil then RepFactionsHolder[RepRewardType] = 0 end

			Dewdrop:AddLine(
				'text', Crayon:Green(RepAmount) .. L[" reputation with "] .. Crayon:Colorize(RGBPercToHex(RepFactionColors[RepFactionsHolder[RepRewardType]].r,
																									RepFactionColors[RepFactionsHolder[RepRewardType]].g,
																									RepFactionColors[RepFactionsHolder[RepRewardType]].b ),
																									BF[RepRewardType]),
				'textHeight', FontIconSize * 12,
				'icon', "Interface\\Addons\\DailiesQuestTracker\\Icons\\Rep\\" .. RepRewardType,
				'iconWidth', FontIconSize * 14,
				'iconHeight', FontIconSize * 14
			)
			if RepFactionsHolder[RepRewardType] == 0 then RepFactionsHolder[RepRewardType] = nil end
			end
		end
		
		-- If GOLD awarded, add it to the tooltip
		if DailyQuestsList[QuestTitleValue]["gold"] then
			Dewdrop:AddLine(
				'text', Abacus:FormatMoneyExtended(DailyQuestsList[QuestTitleValue]["gold"], true),
				'textHeight', FontIconSize * 12, 
				'icon', "Interface\\Icons\\INV_Misc_Coin_02",
				'iconWidth', FontIconSize * 14,
				'iconHeight', FontIconSize * 14
	        )
		end
    end
	)
	
	-- Rewards Tooltip placement
	Dewdrop20Level1:ClearAllPoints();
	if self.db.char.TooltipPlacement == "Left" then
		Dewdrop20Level1:SetPoint("TOPLEFT", GetMouseFocus():GetName(), "TOPRIGHT", 10, 30)
	else
		Dewdrop20Level1:SetPoint("TOPRIGHT", GetMouseFocus():GetName(), "TOPLEFT", -10, 30)
	end
	Dewdrop20Level1:Show();
end

-- Click on FuBar icon will toggle the Quest Log Window
function Dailies:OnClick(button)
	ToggleQuestLog();
end

-- Makes the tooltip an actual item
function Dailies:SetTooltip(arg1, arg2)
	GameTooltip_SetDefaultAnchor(GameTooltip, this)
	GameTooltip:SetHyperlink("item:"..arg1..":0:0:0:0:0:0:0")
end

-- Click to paste the quest link into the chatbox
function Dailies:QuestClickLink(link, prefix)
	_, _, _, _, _, _, _, QuestID = Quixote:GetQuestByName(link)
	
	if not ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Show()
	end
	
	if DailyQuestsList[link]["loc"] then
		if QuestID then
			QuestLink = GetQuestLink(QuestID)
			ChatFrameEditBox:Insert(prefix .. QuestLink .. L[" located in "].. DailyQuestsList[link]["loc"])
		else
			ChatFrameEditBox:Insert(prefix .. link .. L[" located in "].. DailyQuestsList[link]["loc"])
		end
	else
		if QuestID then
			QuestLink = GetQuestLink(QuestID)
			ChatFrameEditBox:Insert(prefix .. QuestLink)
		else
			ChatFrameEditBox:Insert(prefix .. link)
		end
	end
end

-- Click to paste itemlink into the chatbox
function Dailies:itemClickLink(link, amount)

	if not ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Show()
	end
	
	if link then
		ChatFrameEditBox:Insert(link .. amount)
	end
end

function Dailies:GOSSIP_SHOW()
	local QuestName, _, _ = GetGossipAvailableQuests()
	if QuestName and DailyQuestsList[QuestName] then
		Dailies:Debug(L["Gathered quest: "] .. QuestName)
		if DailyQuestsList[QuestName]["group"] == "HInstance" then DailyHInstance = QuestName; DailiesKnown = true; DailiesCommHolder["HInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "NInstance" then DailyNInstance = QuestName; DailiesKnown = true; DailiesCommHolder["NInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Cooking" then DailyCooking = QuestName; DailiesKnown = true; DailiesCommHolder["Cooking"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Fishing" then DailyFishing = QuestName; DailiesKnown = true; DailiesCommHolder["Fishing"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvP" then DailyBGPvP = QuestName; DailiesKnown = true; DailiesCommHolder["BGPvP"] = QuestName end
	end
end

function Dailies:QUEST_DETAIL()
	local QuestName = GetTitleText()
	if QuestName and DailyQuestsList[QuestName] then
		Dailies:Debug(L["Gathered quest: "] .. QuestName)
		if DailyQuestsList[QuestName]["group"] == "HInstance" then DailyHInstance = QuestName; DailiesKnown = true; DailiesCommHolder["HInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "NInstance" then DailyNInstance = QuestName; DailiesKnown = true; DailiesCommHolder["NInstance"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Cooking" then DailyCooking = QuestName; DailiesKnown = true; DailiesCommHolder["Cooking"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "Fishing" then DailyFishing = QuestName; DailiesKnown = true; DailiesCommHolder["Fishing"] = QuestName end
		if DailyQuestsList[QuestName]["group"] == "BGPvP" then DailyBGPvP = QuestName; DailiesKnown = true; DailiesCommHolder["BGPvP"] = QuestName end
	end
end